var generators = require('yeoman-generator');

/**
 *
 */
module.exports = generators.Base.extend({
	prompting: function () {
		var done = this.async();
		var d = new Date();
		var projectTypes = {
			'com': 'Component',
			'lib': 'Library',
			'mod': 'Module',
			'plg': 'Plugin',
			'tpl': 'Template'
		};
		this.licenses = {
			'GNU General Public License, version 2 or later': 'GPL2'
		};

		// Initialize empty data
		this.data = {};

		// List all prompts
		var prompts = [
			{
				type: 'list',
				name: 'project',
				choices: this._getObjectValues(projectTypes),
				message: 'What\'s type of project you\'re about to create?'
			},
			{
				type: 'list',
				name: 'group',
				message: 'Which is the type of the plugin?',
				choices: [
					'custom',
					'authentication',
					'captcha',
					'content',
					'editors',
					'editors-xtd',
					'extension',
					'finder',
					'installer',
					'quickicon',
					'search',
					'system',
					'twofactorauth',
					'user'
				],
				when: function (answers) {
					return answers.project == projectTypes['plg'];
				},
				default: 'custom'
			},
			{
				type: 'input',
				name: 'name',
				message: 'What\'s the name of the project?',
				validate: function (answer) {
					if (answer.match(/^(com|lib|mod|pkg|plg|tpl)_/)) {
						return 'Don\'t write com_, lib_, mod_, plg_[group] and tpl_ prefixes.';
					}
					else if (!answer.match(/./)) {
						return false;
					}

					return true;
				}
			},
			{
				type: 'input',
				name: 'description',
				message: 'A description for that project?',
				default: 'TODO'
			},
			{
				type: 'input',
				name: 'author',
				message: 'Who is the author?',
				validate: function (answer) {
					return answer.length > 0;
				},
				store: true
			},
			{
				type: 'input',
				name: 'authorEmail',
				message: 'Which is the author\'s email?',
				validate: function (answer) {
					if (!answer.match(/([a-z0-9_\.-]+)@([\da-z\.-]+)\.([a-z\.]{2,6})/)) {
						return 'Please provide a valid email address.';
					}

					return true;
				},
				store: true
			},
			{
				type: 'input',
				name: 'authorUrl',
				message: 'Which is the author\'s website?',
				validate: function (answer) {
					if (!answer.match(/(https?:\/\/)?([\da-z\.-]+)\.([a-z\.]{2,6})([\/\w \.-]*)*\/?/)) {
						return 'Please provide a valid url address.';
					}

					return true;
				},
				store: true
			},
			{
				type: 'input',
				name: 'languageTag',
				message: 'Which is the default language?',
				validate: function (answer) {
					if (!answer.match(/\w{2}-[A-Z]{2}/)) {
						return 'Please provide a compliant language tag (ex. en-GB).';
					}

					return true;
				},
				when: function (answers) {
					return answers.project != projectTypes['lib'];
				},
				default: 'en-GB'
			},
			{
				type: 'list',
				name: 'license',
				message: 'How do you license the project?',
				choices: Object.keys(this.licenses),
				store: true
			},
			{
				type: 'list',
				name: 'updateServerType',
				message: 'Your have an update server?',
				choices: [
					'none',
					'collection',
					'extension'
				],
				when: function (answers) {
					return answers.project == projectTypes['com'];
				},
				default: 'none'
			},
			{
				type: 'input',
				name: 'updateServerName',
				message: 'What\'s the update server name?',
				when: function (answers) {
					return answers.updateServerType !== undefined && answers.updateServerType != 'none';
				},
				validate: function (answer) {
					return answer.length > 0;
				},
				store: true
			},
			{
				type: 'input',
				name: 'updateServerUrl',
				message: 'What\'s the update server name?',
				when: function (answers) {
					return answers.updateServerType !== undefined && answers.updateServerType != 'none';
				},
				validate: function (answer) {
					if (!answer.match(/(https?:\/\/)?([\da-z\.-]+)\.([a-z\.]{2,6})([\/\w \.-]*)*\/?/)) {
						return 'Please provide a valid url address.';
					}

					return true;
				},
				store: true
			}
		];

		// Prompt and bind answers
		this.prompt(prompts, function (answers) {
			this.data.author = answers.author;
			this.data.authorEmail = answers.authorEmail;
			this.data.authorUrl = answers.authorUrl;
			this.data.description = answers.description;
			this.data.group = answers.group;
			this.data.languageTag = answers.languageTag;
			this.data.license = answers.license;

			this.data.project = answers.project;
			this.data.realName = answers.name;
			this.data.realNameLower = answers.name.toLowerCase();
			this.data.realNameUpper = answers.name.toUpperCase();
			this.data.prefix = projectTypes.getKeyByValue(answers.project);

			if (answers.group) {
				this.data.fullNameLower = this.data.prefix + '_' + answers.group + '_' + this.data.realNameLower;
				this.data.fullNameCamelized = this.data.prefix.capitalize() + answers.group.capitalize() + this.data.realName.capitalize();
				this.data.groupCapitalized = answers.group.capitalize();
			} else {
				this.data.fullNameLower = this.data.prefix + '_' + this.data.realNameLower;
				this.data.fullNameCamelized = this.data.prefix.capitalize() + this.data.realName.capitalize();
			}

			this.data.fullNameUpper = this.data.fullNameLower.toUpperCase();

			this.data.updateServerName = answers.updateServerName;
			this.data.updateServerType = answers.updateServerType;
			this.data.updateServerUrl = answers.updateServerUrl;

			done();
		}.bind(this));

		// Build the date for the manifest creation date
		this.data.creationDate = d.getDate() + ' ' + d.toLocaleString('en-us', {month: "long"}) + ' ' + d.getFullYear();
		this.data.creationYear = d.getFullYear();
	},
	writing: function () {
		var source = this.data.project.toLowerCase();
		var root = this.sourceRoot() + '/' + source;
		var files = this.expandFiles('**', {dot: true, cwd: root});

		// Add the license
		this.copy('licenses/' + this.licenses[this.data.license], 'source/LICENSE.txt');
		this.data.license += '; see LICENSE.txt';

		// Parse the template files
		for (var i in files) {
			if (files.hasOwnProperty(i)) {
				var outputName = this._replacePlaceholders(files[i]);

				if (this.data.group) {
					var manifest = files[i].match(/__realNameLower__\.([^.]+)\.php$/);

					// Skip wrong group variant files
					if (manifest) {
						if (this.data.group != manifest[1]) {
							continue;
						}
						else {
							outputName = outputName.replace('.' + this.data.group, '');
						}
					}
				}

				// Copy the file
				this.copy(source + '/' + files[i], outputName, this._replacePlaceholders.bind(this));
			}
		}

		// Copy common stuff
		this.copy('.gitignore', '.gitignore');
		this.copy('bower.json', 'bower.json', this._replacePlaceholders.bind(this));
		this.copy('CHANGELOG.md', 'CHANGELOG.md');
		this.copy('composer.json', 'composer.json');
		this.copy('Gruntfile.js', 'Gruntfile.js');
		this.copy('package.json', 'package.json', this._replacePlaceholders.bind(this));
	},
	/**
	 *
	 * @param contents
	 * @private
	 */
	_replacePlaceholders: function (contents) {
		var newContents = contents;

		for (placeholder in this.data) {
			if (this.data.hasOwnProperty(placeholder)) {
				var regexp = new RegExp('__' + placeholder + '__', 'g');
				newContents = newContents.replace(regexp, this.data[placeholder]);
			}
		}

		return newContents;
	},
	/**
	 * Returns the values of an objects, like associative arrays.
	 *
	 * @param obj The object to parse.
	 * @returns {Array} The list of the values.
	 * @private
	 */
	_getObjectValues: function (obj) {
		var values = [];

		for (var key in obj) {
			if (obj.hasOwnProperty(key)) {
				values.push(obj[key]);
			}
		}

		return values;
	}
});

/**
 * Return a string with the first letter capitalized.
 *
 * @returns {string}
 */
String.prototype.capitalize = function () {
	return this.charAt(0).toUpperCase() + this.slice(1);
};

/**
 *
 * @param value
 * @returns {string}
 */
Object.prototype.getKeyByValue = function (value) {
	for (var prop in this) {
		if (this.hasOwnProperty(prop)) {
			if (this[prop] === value) {
				return prop;
			}
		}
	}
};